@@ -570,10 +570,10 @@ static int print_min_dev_size(int fd, u64 devid)
 	sk = btrfs_tree_search_sk(&args);
 	sk->tree_id = BTRFS_DEV_TREE_OBJECTID;
 	sk->min_objectid = devid;
-	sk->max_objectid = devid;
-	sk->max_type = BTRFS_DEV_EXTENT_KEY;
 	sk->min_type = BTRFS_DEV_EXTENT_KEY;
 	sk->min_offset = 0;
+	sk->max_objectid = devid;
+	sk->max_type = BTRFS_DEV_EXTENT_KEY;
 	sk->max_offset = (u64)-1;
 	sk->min_transid = 0;
 	sk->max_transid = (u64)-1;
@@ -980,10 +980,10 @@ static u64 fill_usage(int fd, u64 lstart)
 	sk = btrfs_tree_search_sk(&args);
 	sk->tree_id = BTRFS_EXTENT_TREE_OBJECTID;
 	sk->min_objectid = lstart;
-	sk->max_objectid = lstart;
 	sk->min_type = BTRFS_BLOCK_GROUP_ITEM_KEY;
-	sk->max_type = BTRFS_BLOCK_GROUP_ITEM_KEY;
 	sk->min_offset = 0;
+	sk->max_objectid = lstart;
+	sk->max_type = BTRFS_BLOCK_GROUP_ITEM_KEY;
 	sk->max_offset = (u64)-1;
 	sk->max_transid = (u64)-1;
 	sk->nr_items = 1;
@@ -1074,21 +1074,23 @@ static int cmd_inspect_list_chunks(const struct cmd_struct *cmd,
 	if (!ctx.stats) {
 		ret = 1;
 		error_msg(ERROR_MSG_MEMORY, NULL);
-		goto out_nomem;
+		goto out;
 	}
 
 	path = argv[optind];
 
 	fd = btrfs_open_file_or_dir(path);
-	if (fd < 0)
-		return 1;
+	if (fd < 0) {
+		ret = 1;
+		goto out;
+	}
 
 	memset(&args, 0, sizeof(args));
 	sk = btrfs_tree_search_sk(&args);
 	sk->tree_id = BTRFS_CHUNK_TREE_OBJECTID;
 	sk->min_objectid = BTRFS_FIRST_CHUNK_TREE_OBJECTID;
-	sk->max_objectid = BTRFS_FIRST_CHUNK_TREE_OBJECTID;
 	sk->min_type = BTRFS_CHUNK_ITEM_KEY;
+	sk->max_objectid = BTRFS_FIRST_CHUNK_TREE_OBJECTID;
 	sk->max_type = BTRFS_CHUNK_ITEM_KEY;
 	sk->max_offset = (u64)-1;
 	sk->max_transid = (u64)-1;
@@ -1096,7 +1098,7 @@ static int cmd_inspect_list_chunks(const struct cmd_struct *cmd,
 	if (!lnumber) {
 		ret = 1;
 		error_msg(ERROR_MSG_MEMORY, NULL);
-		goto out_nomem;
+		goto out;
 	}
 
 	while (1) {
@@ -1104,7 +1106,7 @@ static int cmd_inspect_list_chunks(const struct cmd_struct *cmd,
 		ret = btrfs_tree_search_ioctl(fd, &args);
 		if (ret < 0) {
 			error("cannot perform the search: %m");
-			return 1;
+			goto out;
 		}
 		if (sk->nr_items == 0)
 			break;
@@ -1144,7 +1146,7 @@ static int cmd_inspect_list_chunks(const struct cmd_struct *cmd,
 					if (!tmp) {
 						ret = 1;
 						error_msg(ERROR_MSG_MEMORY, NULL);
-						goto out_nomem;
+						goto out;
 					}
 					memcpy(tmp, lnumber, sizeof(u64) * old_size);
 					lnumber = tmp;
@@ -1167,7 +1169,7 @@ static int cmd_inspect_list_chunks(const struct cmd_struct *cmd,
 					if (!ctx.stats) {
 						ret = 1;
 						error_msg(ERROR_MSG_MEMORY, NULL);
-						goto out_nomem;
+						goto out;
 					}
 				}
 			}
@@ -1185,7 +1187,7 @@ static int cmd_inspect_list_chunks(const struct cmd_struct *cmd,
 	ret = print_list_chunks(&ctx, sortmode, unit_mode, with_usage, with_empty);
 	close(fd);
 
-out_nomem:
+out:
 	free(ctx.stats);
 	free(lnumber);
 
@@ -1294,13 +1296,12 @@ static int read_chunk_tree(int fd, struct chunk **chunks, size_t *num_chunks)
 
 		chunk = &(*chunks)[*num_chunks];
 		chunk->offset = sh.offset;
-		chunk->length = le64_to_cpu(item->length);
-		chunk->stripe_len = le64_to_cpu(item->stripe_len);
-		chunk->type = le64_to_cpu(item->type);
-		chunk->num_stripes = le16_to_cpu(item->num_stripes);
-		chunk->sub_stripes = le16_to_cpu(item->sub_stripes);
-		chunk->stripes = calloc(chunk->num_stripes,
-					sizeof(*chunk->stripes));
+		chunk->length = get_unaligned_le64(&item->length);
+		chunk->stripe_len = get_unaligned_le64(&item->stripe_len);
+		chunk->type = get_unaligned_le64(&item->type);
+		chunk->num_stripes = get_unaligned_le16(&item->num_stripes);
+		chunk->sub_stripes = get_unaligned_le16(&item->sub_stripes);
+		chunk->stripes = calloc(chunk->num_stripes, sizeof(*chunk->stripes));
 		if (!chunk->stripes) {
 			perror("calloc");
 			return -1;
@@ -1311,8 +1312,8 @@ static int read_chunk_tree(int fd, struct chunk **chunks, size_t *num_chunks)
 			const struct btrfs_stripe *stripe;
 
 			stripe = &item->stripe + i;
-			chunk->stripes[i].devid = le64_to_cpu(stripe->devid);
-			chunk->stripes[i].offset = le64_to_cpu(stripe->offset);
+			chunk->stripes[i].devid = get_unaligned_le64(&stripe->devid);
+			chunk->stripes[i].offset = get_unaligned_le64(&stripe->offset);
 		}
 
 next:
@@ -1423,7 +1424,7 @@ static int map_physical_start(int fd, struct chunk *chunks, size_t num_chunks,
 		type = item->type;
 		if (type == BTRFS_FILE_EXTENT_REG ||
 		    type == BTRFS_FILE_EXTENT_PREALLOC) {
-			logical_offset = le64_to_cpu(item->disk_bytenr);
+			logical_offset = get_unaligned_le64(&item->disk_bytenr);
 			if (logical_offset) {
 				/* Regular extent */
 				chunk = find_chunk(chunks, num_chunks, logical_offset);
@@ -1457,7 +1458,7 @@ static int map_physical_start(int fd, struct chunk *chunks, size_t num_chunks,
 			goto out;
 		}
 		if (item->other_encoding != 0) {
-			error("file with other_encoding: %u", le16_to_cpu(item->other_encoding));
+			error("file with other_encoding: %u", get_unaligned_le16(&item->other_encoding));
 			ret = -EINVAL;
 			goto out;
 		}
